﻿using Borepin.Base;
using Prism.Commands;
using Prism.Navigation;
using System.Windows.Input;
using FabAccessAPI.Schema;
using Prism.Services;
using Borepin.Service;
using System.Threading.Tasks;
using Borepin.Base.Exceptions;
using NFC.Interfaces;
using System.Collections.Generic;
using Borepin.Model;

namespace Borepin.PageModel
{
    public class CreateCardPageModel : ConnectionModelBase
    {
        #region Private Fields
        private CardConfig _CardConfig;
        private User _User;
        private INFCService _NFCService;
        #endregion

        #region Contructors
        public CreateCardPageModel(INavigationService navigationService, IPageDialogService pageDialogService, IAPIService apiService, INFCService nfcService) : base(navigationService, pageDialogService, apiService)
        {
            _NFCService = nfcService;
            _CardConfig= new CardConfig();

            CreateCardCommand = new DelegateCommand(CreateCardCommandExecute);
            
            ScanPICCKeyCommand = new DelegateCommand(ScanPICCKeyCommandExecute);
            RandomPICCKeyCommand = new DelegateCommand(RandomPICCKeyCommandExecute);

            ScanAPPKeyCommand = new DelegateCommand(ScanAPPKeyCommandExecute);
            RandomAPPKeyCommand = new DelegateCommand(RandomAPPKeyCommandExecute);

            RefreshCommand = new DelegateCommand(async () => await RefreshCommandExecute().ConfigureAwait(true));
        }
        #endregion

        #region Data
        public override Task LoadInstance(object instance)
        {
            if (instance is string)
            {
                _CardConfig.UserID  = instance as string;
            }
            else
            {
                throw new InstanceIncorrectException();
            }

            return Task.CompletedTask;
        }

        public override async Task LoadAPIData()
        {
            _User = (await _API.Session.UserSystem.Search.GetUserByName(_CardConfig.UserID).ConfigureAwait(false)).Just;

            ReaderIDs = await Task.Run(() => 
            {
                return _NFCService.GetReaderIDs();
            }).ConfigureAwait(false);

            if (ReaderIDs.Count > 0)
            {
                ReaderID = ReaderIDs[0];
            }

            PICCKey = _CardConfig.ConvertToString(_CardConfig.PICCKey);
            APPKey = _CardConfig.ConvertToString(_CardConfig.APPKey);
        }

        public override Task<object> CreateInstance()
        {
            return Task.FromResult<object>(_CardConfig);
        }
        #endregion

        #region Fields
        private IList<string> _ReaderIDs;
        public IList<string> ReaderIDs
        {
            get => _ReaderIDs;
            set => SetProperty(ref _ReaderIDs, value);
        }

        private string _ReaderID;
        public string ReaderID
        {
            get => _ReaderID;
            set => SetProperty(ref _ReaderID, value);
        }

        private string _PICCKey;
        public string PICCKey
        {
            get => _PICCKey;
            set => SetProperty(ref _PICCKey, value);
        }

        private string _APPKey;
        public string APPKey
        {
            get => _APPKey;
            set => SetProperty(ref _APPKey, value);
        }

        private string _FormatCard;
        public string FormatCard
        {
            get => _FormatCard;
            set => SetProperty(ref _FormatCard, value);
        }
        #endregion

        #region Commands
        private ICommand _RefreshCommand;
        public ICommand RefreshCommand
        {
            get => _RefreshCommand;
            set => SetProperty(ref _RefreshCommand, value);
        }
        public async Task RefreshCommandExecute()
        {
            if (_API.IsConnected)
            {
                try
                {
                    await LoadAPIData().ConfigureAwait(true);
                }
                catch (System.Exception)
                {
                    // TODO
                }
            }
        }

        private ICommand _CreateCardCommand;

        public ICommand CreateCardCommand
        {
            get => _CreateCardCommand;
            set => SetProperty(ref _CreateCardCommand, value);
        }

        public async void CreateCardCommandExecute()
        {
            IsBusy = true;

            await Task.Delay(1000).ConfigureAwait(false);

            IsBusy = false;
        }

        #region PICCKey
        private ICommand _ScanPICCKeyCommand;

        public ICommand ScanPICCKeyCommand
        {
            get => _ScanPICCKeyCommand;
            set => SetProperty(ref _ScanPICCKeyCommand, value);
        }

        public async void ScanPICCKeyCommandExecute()
        {
            IsBusy = true;

            await Task.Delay(1000).ConfigureAwait(false);

            IsBusy = false;
        }

        private ICommand _RandomPICCKeyCommand;

        public ICommand RandomPICCKeyCommand
        {
            get => _RandomPICCKeyCommand;
            set => SetProperty(ref _RandomPICCKeyCommand, value);
        }

        public void RandomPICCKeyCommandExecute()
        {
            _CardConfig.PICCKey = _CardConfig.GenerateRandomKey();
            PICCKey = _CardConfig.ConvertToString(_CardConfig.PICCKey);
        }

        private ICommand _ExportPICCKeyCommand;

        public ICommand ExportPICCKeyCommand
        {
            get => _ExportPICCKeyCommand;
            set => SetProperty(ref _ExportPICCKeyCommand, value);
        }

        public async void ExportPICCKeyCommandExecute()
        {
            IsBusy = true;

            await Task.Delay(1000).ConfigureAwait(false);

            IsBusy = false;
        }
        #endregion

        #region APPKey
        private ICommand _ScanAPPKeyCommand;

        public ICommand ScanAPPKeyCommand
        {
            get => _ScanAPPKeyCommand;
            set => SetProperty(ref _ScanAPPKeyCommand, value);
        }

        public async void ScanAPPKeyCommandExecute()
        {
            IsBusy = true;

            await Task.Delay(1000).ConfigureAwait(false);

            IsBusy = false;
        }

        private ICommand _RandomAPPKeyCommand;

        public ICommand RandomAPPKeyCommand
        {
            get => _RandomAPPKeyCommand;
            set => SetProperty(ref _RandomAPPKeyCommand, value);
        }

        public void RandomAPPKeyCommandExecute()
        {
            _CardConfig.APPKey = _CardConfig.GenerateRandomKey();
            APPKey = _CardConfig.ConvertToString(_CardConfig.APPKey);
        }

        private ICommand _ExportAPPKeyCommand;

        public ICommand ExportAPPKeyCommand
        {
            get => _ExportAPPKeyCommand;
            set => SetProperty(ref _ExportAPPKeyCommand, value);
        }

        public async void ExportAPPKeyCommandExecute()
        {
            IsBusy = true;

            await Task.Delay(1000).ConfigureAwait(false);

            IsBusy = false;
        }
        #endregion

        #endregion
    }
}
